@@ -47,7 +47,7 @@
 static struct intel_device_info *chipset_info;
 
 static const struct intel_device_info intel_generic_info = {
-	.gen = 0,
+	.gen = -1,
 };
 
 static const struct intel_device_info intel_i81x_info = {
@@ -154,7 +154,7 @@ static const SymTabRec _intel_chipsets[] = {
 SymTabRec *intel_chipsets = (SymTabRec *) _intel_chipsets;
 
 #define INTEL_DEVICE_MATCH(d,i) \
-    { 0x8086, (d), PCI_MATCH_ANY, PCI_MATCH_ANY, 0, 0, (intptr_t)(i) }
+    { 0x8086, (d), PCI_MATCH_ANY, PCI_MATCH_ANY, 0x3 << 16, 0xff << 16, (intptr_t)(i) }
 
 static const struct pci_id_match intel_device_match[] = {
 #if !KMS_ONLY
@@ -223,29 +223,38 @@ static const struct pci_id_match intel_device_match[] = {
 	{ 0, 0, 0 },
 };
 
-void intel_detect_chipset(ScrnInfoPtr scrn,
-			  struct pci_device *pci,
-			  struct intel_chipset *chipset)
+const struct intel_device_info *
+intel_detect_chipset(ScrnInfoPtr scrn,
+		     EntityInfoPtr ent, struct pci_device *pci)
 {
+	MessageType from = X_PROBED;
+	const char *name = NULL;
 	int i;
 
-	chipset->info = chipset_info;
-	chipset->name = NULL;
+	if (ent->device->chipID >= 0) {
+		xf86DrvMsg(scrn->scrnIndex, from = X_CONFIG,
+			   "ChipID override: 0x%04X\n",
+			   ent->device->chipID);
+		DEVICE_ID(pci) = ent->device->chipID;
+	}
 
 	for (i = 0; intel_chipsets[i].name != NULL; i++) {
 		if (DEVICE_ID(pci) == intel_chipsets[i].token) {
-			chipset->name = intel_chipsets[i].name;
+			name = intel_chipsets[i].name;
 			break;
 		}
 	}
-	if (chipset->name == NULL) {
+	if (name == NULL) {
 		xf86DrvMsg(scrn->scrnIndex, X_WARNING, "unknown chipset\n");
-		chipset->name = "unknown";
+		name = "unknown";
 	} else {
-		xf86DrvMsg(scrn->scrnIndex, X_INFO,
+		xf86DrvMsg(scrn->scrnIndex, from,
 			   "Integrated Graphics Chipset: Intel(R) %s\n",
-			   chipset->name);
+			   name);
 	}
+
+	scrn->chipset = name;
+	return chipset_info;
 }
 
 /*
